==> aloe-1.4.tar.gz: aloe-1.4/hw_api/lnx/src/parse_str.c <==
/*
 * parse_str.c
 *
 * Copyright (c) 2009 Ismael Gomez-Miguelez, UPC <ismael.gomez at tsc.upc.edu>,
 *                    Xavier Reves, UPC <xavier.reves at tsc.upc.edu>
 * All rights reserved.
 *
 *
 * This file is part of ALOE.
 *
 * ALOE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ALOE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ALOE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


/** @file parse_str.c
 * @brief ALOE Launcher for Linux Parsing Functions
 *
 * This file implements functions called to parse the following
 * files need to launch the ALOE Linux Launcher.
 *     	- Daemon Configuration File: Describes the daemons that will
 * 		be launched.
 * 	- Interface Configuration File: Describes the external physical
 *		interfaces (port, address, r/w modes, etc.)
 *	- CPU Configuration File: Describes the local processor characteristics.
 */

#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>
#include <memory.h>
#include <dirent.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>
#include <setjmp.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <sched.h>
#include <string.h>

/********************************/
/****   LOCAL INCLUDES       ****/
/********************************/

#define PARSE_LINE_LEN	163

#include "mem.h"
#include "phal_hw_api.h"
#include "hwapi_backd.h"
#include "parse_str.h"
#include "parser.h"
#include "hwapi_netd.h"

/** @defgroup parse ALOE Launcher for Linux Parsing Functions
 *
 * This functions parse the ALOE Launcher Configuration files
 * need to launch ALOE main process.
 *
 * @{
 */



/** Interface Configuration Parameters
 */
char *itf_parameters[NOF_ITF_PARAMS + 1];

/** Configure Interface Config file parameters
 */
void init_itf_param_strings(void)
{
    itf_parameters[ITF_ID] = "id"; /* Id of the Interface */
    itf_parameters[ITF_PORT] = "port"; /* TCP local/remote port */
    itf_parameters[ITF_ADDRESS] = "address"; /* IP address */
    itf_parameters[ITF_MODE] = "mode"; /* "in" or "out" */
    itf_parameters[ITF_BW] = "bw"; /* float, in bps */
    itf_parameters[ITF_DELAY] = "delay"; /* int, in milisec */
    itf_parameters[ITF_TYPE] = "type";
    itf_parameters[NOF_ITF_PARAMS] = "\0";
}

/** Interface Configuration Parsing Function.
 *
 * Actually parses the file and fills ext_itf structure with parsed data.
 *
 * @sa init_itf_param_string
 *
 * @param f FILE pointer
 * @param ext_itf Pointer to first element of the ext_itf database
 * @param max_itf Size in number of elements of the database
 *
 * @return >=0 Number of interfaces parsed
 * @return <0 Error parsing
 */
int parse_itf_cfg(FILE *f, struct ext_itf *ext_itf, int max_itf)
{

    int file_len;
    char string[PARSE_LINE_LEN];
    int nof_itf;
    int error;
    int next_section, i, n;
    char *buffer, *pname, *pvalue;
    int line;
    int aux;

    init_itf_param_strings();

    file_len = read_file(f, &buffer, 0);
    if (file_len < 0) {
        printf("ERROR: There were errors reading config file\n");
        return (-1);
    }

    i = 0;
    nof_itf = 0;
    while (i < file_len && nof_itf < max_itf) {
        n = read_line(buffer, i, string);
        i += n;
        if (strcmp(string, "[xitf]") == 0) {
            line = 0;
            next_section = 0;
            error = 0;
            while ((!next_section) && (i < file_len)) {
                n = read_line(buffer, i, string);
                if (string[0] == '[')
                    next_section = 1;
                else {
                    i += n;
                    line++;
                    pname = string;
                    pvalue = strchr(string, '=');
                    pvalue[0] = '\0';
                    pvalue++;
                    switch (param_name_start(pname, itf_parameters)) {
                    case ITF_ADDRESS:
                        if (strlen(pvalue) < 16) {
                            strcpy(ext_itf[nof_itf].address, pvalue);
                        } else {
                            printf("Error parsing xitf file. Address to long.\n");
                            error = 1;
                            break;
                        }
                        break;
                    case ITF_ID:
                        sscanf(pvalue, "0x%x", &aux);
                        ext_itf[nof_itf].id = (xitfid_t) aux;
                        ext_itf[nof_itf].info.id = (xitfid_t) aux;
                        break;
                    case ITF_TYPE:
                        if (!strcmp(pvalue,"tcp")) {
                            ext_itf[nof_itf].type=NET_TCP;
                        } else if (!strcmp(pvalue,"udp")) {
                            ext_itf[nof_itf].type=NET_UDP;
                        } else {
                            printf("Error parsing xitf file. Unknown type %s\n",pvalue);
                            error=1;
                            break;
                        }
                        break;
                    case ITF_MODE:
                        if (!strcmp(pvalue, "out"))
                            ext_itf[nof_itf].mode = FLOW_WRITE_ONLY;
                        else if (!strcmp(pvalue, "in"))
                            ext_itf[nof_itf].mode = FLOW_READ_ONLY;
                        else if (!strcmp(pvalue, "inout"))
                            ext_itf[nof_itf].mode = FLOW_READ_WRITE;
                        else if (!strcmp(pvalue, "outin"))
                            ext_itf[nof_itf].mode = FLOW_WRITE_READ;
                        else {
                            printf("Error parsing xitf file. Mode must be in/out.\n");
                            error = 1;
                            break;
                        }
                        ext_itf[nof_itf].info.mode = ext_itf[nof_itf].mode;
                        break;
                    case ITF_PORT:
                        sscanf(pvalue, "%d", &aux);
                        ext_itf[nof_itf].port = aux;
                        break;
                    case ITF_DELAY:
                        ext_itf[nof_itf].info.delay = atof(pvalue);
                        break;
                    case ITF_BW:
                        ext_itf[nof_itf].info.BW = atof(pvalue);
                        break;
                    default:
                        printf("Error parsing xitf file: Unknown parameter\n");
                        error = 1;
                        break;
                    }

                    if (error) {
                        printf("Error parsing xitf file. Itf %d, parameter %d\n", nof_itf + 1, line);
                        return -1;
                    }
                }
            }
            //			ext_itf[nof_itf].type=NET_TCP;
            nof_itf++;
            if (nof_itf == max_itf)
                printf("CAUTION: Maximum interfaces exceed\n");
        }
    }
    if (error) {
        return -1;
    } else
        return nof_itf;
}

/** CPU Configuration Parameters
 */
char *cpu_parameters[NOF_CPU_PARAMS + 1];

int read_daemons(char *pvalue, struct launch_daemons *daemons, int max_d)  {
    char *x,*y;
    int i;

    /** read comma-sepparated values */
    x=pvalue;
    i=0;
    if (x=='\0') {
        return 0;
    }
    do {
        y=x;
        x=strstr(y,",");
        if (x) {
            *x='\0';
            x++;
        }
        strcpy(daemons[i].path,y);
        i++;
    } while(x && i<max_d);

    return i;
}

/** Daemon Configuration Parameters configuration.
 *
 * Configures the parameters accepted in the daemon configuration file
 *
 * @todo Separate CPU parsing from daemon parsing. More CPU parameters?
 * @todo Replace name by daemon code
 *
 */
void init_daemon_param_strings(void)
{
    cpu_parameters[CPU_NAME] = "name";
    cpu_parameters[CPU_MIPS] = "macs"; 
    cpu_parameters[CPU_TSLOT] = "tslot";
    cpu_parameters[CPU_DAEMON] = "run_as_daemon";
    cpu_parameters[CPU_OUTPUT] = "output_file";
    cpu_parameters[CPU_DEBUG] = "debug_mode";
    cpu_parameters[CPU_RESPATH] = "work_path";
    cpu_parameters[CPU_LOCK] = "lock_file";
    cpu_parameters[CPU_FULLBRIDGE] = "full_bridge";
    cpu_parameters[CPU_ENDIAN] = "endian";
    cpu_parameters[CPU_CORES] = "nof_cores";
    cpu_parameters[CPU_BTATEXIT] = "printbt_atexit";
    cpu_parameters[CPU_BTATTERM] = "printbt_atrtfault";
    cpu_parameters[CPU_DLIST] = "daemons";
    cpu_parameters[CPU_ITFDELAY] = "itfs_delay";
    cpu_parameters[CPU_ARCH] = "arch";
    cpu_parameters[CPU_FLUSH] = "enable_fflush";
    cpu_parameters[CPU_NSAMPLES] = "numsamples";
    cpu_parameters[CPU_ENABLEDAC] = "enable_dac";
    cpu_parameters[NOF_CPU_PARAMS] = "\0";
}

/** Daemon & CPU Configuration Parsing Function.
 *
 * Actually parses the file and fills daemon_db structure with parsed data.
 *
 * @sa init_daemon_param_string
 *
 * @param f FILE pointer
 * @param daemon_db Pointer to first element of the daemon_db database
 * @param cpu_info Pointer to cpu configuration structure
 * @param max_d Size in number of elements of the database
 *
 * @return >=0 Number of daemons parsed
 * @return <0 Error parsing
 */
int parse_cpu_cfg(FILE *f, struct launch_daemons *daemon_db, int max_d, struct hw_api_db *hwapi_db)
{

    int file_len;
    char string[PARSE_LINE_LEN];
    int nof_d;
    int error;
    int next_section, i, n;
    char *buffer, *pname, *pvalue;
    int line;

    init_daemon_param_strings();

    file_len = read_file(f, &buffer, 0);
    if (file_len < 0) {
        printf("ERROR: There were errors reading config file\n");
        return (-1);
    }

    i = 0;
    line = 0;
    error = 0;

    while (i < file_len) {
        n = read_line(buffer, i, string);
        i += n;
        line++;
        pname = string;
        pvalue = strchr(string, '=');
        pvalue[0] = '\0';
        pvalue++;
        switch (param_name_start(pname, cpu_parameters)) {
        case CPU_NAME:
            if (strnlen(pvalue, CPU_NAME_LEN) < CPU_NAME_LEN) {
                strncpy(hwapi_db->cpu_info.name, pvalue, CPU_NAME_LEN);
            } else {
                printf("Error parsing platform file. Processor name too long.\n");
                error = 1;
                break;
            }
            break;
        case CPU_BTATEXIT:
            hwapi_db->printbt_atexit = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_BTATTERM:
            hwapi_db->printbt_atterm = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_ARCH:
            if (!strncmp(pvalue,"x86",3)) {
                hwapi_db->cpu_info.plat_family = PLATFORM_LIN_X86;
            } else if (!strncmp(pvalue,"x86_64",6)) {
                hwapi_db->cpu_info.plat_family = PLATFORM_LIN_X86_64;
            } else if (!strncmp(pvalue,"arm",3)) {
                hwapi_db->cpu_info.plat_family = PLATFORM_LIN_ARM;
            } else if (!strncmp(pvalue,"ppc",3)) {
                hwapi_db->cpu_info.plat_family = PLATFORM_LIN_PPC;
            } else {
                printf("Error parsing platform file. Unknown processor architecture %s\n",pvalue);
                error=1;
                break;
            }
            break;
        case CPU_ITFDELAY:
            hwapi_db->default_delay = atoi(pvalue);
            break;
        case CPU_DEBUG:
            hwapi_db->cpu_info.debug_level = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_FLUSH:
            hwapi_db->flushout = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_NSAMPLES:
            hwapi_db->numsamples = atoi(pvalue);            
            break;
        case CPU_ENABLEDAC:
            hwapi_db->enabledac = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_MIPS:
            hwapi_db->cpu_info.C = atoi(pvalue);
            break;
        case CPU_TSLOT:
            hwapi_db->cpu_info.tslen_usec = atoi(pvalue);
            break;
        case CPU_DAEMON:
            hwapi_db->run_as_daemon = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_OUTPUT:
            strncpy(hwapi_db->output_file, pvalue, DEF_FILE_LEN);
            break;
        case CPU_FULLBRIDGE:
            hwapi_db->net_db.full_bridge = strncmp(pvalue,"yes",3)?0:1;
            break;
        case CPU_ENDIAN:
            hwapi_db->net_db.litendian = strncmp(pvalue,"big",3)?1:0;
            break;
        case CPU_CORES:
            hwapi_db->cpu_info.nof_cores = atoi(pvalue);
            break;
        case CPU_RESPATH:
            strncpy(hwapi_db->res_path, pvalue, DEF_FILE_LEN);
            break;
        case CPU_LOCK:
            strncpy(hwapi_db->lock_file, pvalue, DEF_FILE_LEN);
            break;
        case CPU_DLIST:
            nof_d=read_daemons(pvalue,daemon_db,max_d);
            break;
        default:
            printf("Error parsing daemons file: Unknown parameter %s\n", pname);
            error = 1;
            break;
        }
        if (error) {
            printf("Error Platform's configuration file. Line %d\n", line);
            return -1;
        }
    }
    if (error) {
        return -1;
    } else
        return nof_d;
}

==> aloe-1.4.5.tar.gz: aloe-1.4.5/hw_api/lnx/src/parse_str.c <==
/*
 * parse_str.c
 *
 * Copyright (c) 2009 Ismael Gomez-Miguelez, UPC <ismael.gomez at tsc.upc.edu>,
 *                    Xavier Reves, UPC <xavier.reves at tsc.upc.edu>
 * All rights reserved.
 *
 *
 * This file is part of ALOE.
 *
 * ALOE is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * ALOE is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with ALOE.  If not, see <http://www.gnu.org/licenses/>.
 *
 */


/** @file parse_str.c
 * @brief ALOE Launcher for Linux Parsing Functions
 *
 * This file implements functions called to parse the following
 * files need to launch the ALOE Linux Launcher.
 *     	- Daemon Configuration File: Describes the daemons that will
 * 		be launched.
 * 	- Interface Configuration File: Describes the external physical
 *		interfaces (port, address, r/w modes, etc.)
 *	- CPU Configuration File: Describes the local processor characteristics.
 */

#define _GNU_SOURCE
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/fcntl.h>
#include <sys/stat.h>
#include <sys/param.h>
#include <sys/wait.h>
#include <unistd.h>
#include <errno.h>
#include <memory.h>
#include <dirent.h>
#include <fcntl.h>
#include <limits.h>
#include <unistd.h>
#include <setjmp.h>
#include <signal.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <sched.h>
#include <string.h>

/********************************/
/****   LOCAL INCLUDES       ****/
/********************************/

#define PARSE_LINE_LEN	163

#include "mem.h"
#include "phal_hw_api.h"
#include "hwapi_backd.h"
#include "parse_str.h"
#include "parser.h"
#include "hwapi_netd.h"

/** @defgroup parse ALOE Launcher for Linux Parsing Functions
 *
 * This functions parse the ALOE Launcher Configuration files
 * need to launch ALOE main process.
 *
 * @{
 */



/** Interface Configuration Parameters
 */
char *itf_parameters[NOF_ITF_PARAMS + 1];

/** Configure Interface Config file parameters
 */
void init_itf_param_strings(void)
{
    itf_parameters[ITF_ID] = "id"; /* Id of the Interface */
    itf_parameters[ITF_PORT] = "port"; /* TCP local/remote port */
    itf_parameters[ITF_ADDRESS] = "address"; /* IP address */
    itf_parameters[ITF_MODE] = "mode"; /* "in" or "out" */
    itf_parameters[ITF_BW] = "bw_mbps"; /* float, in bps */
    itf_parameters[ITF_DELAY] = "delay"; /* int, in milisec */
    itf_parameters[ITF_TYPE] = "type";
    itf_parameters[NOF_ITF_PARAMS] = "\0";
}

/** Interface Configuration Parsing Function.
 *
 * Actually parses the file and fills ext_itf structure with parsed data.
 *
 * @sa init_itf_param_string
 *
 * @param f FILE pointer
 * @param ext_itf Pointer to first element of the ext_itf database
 * @param max_itf Size in number of elements of the database
 *
 * @return >=0 Number of interfaces parsed
 * @return <0 Error parsing
 */
int parse_itf_cfg(FILE *f, struct ext_itf *ext_itf, int max_itf)
{

    int file_len;
    char string[PARSE_LINE_LEN];
    int nof_itf;
    int error;
    int next_section, i, n;
    char *buffer, *pname, *pvalue;
    int line;
    int aux;

    init_itf_param_strings();

    file_len = read_file(f, &buffer, 0);
    if (file_len < 0) {
        printf("ERROR: There were errors reading config file\n");
        return (-1);
    }

    i = 0;
    nof_itf = 0;
    while (i < file_len && nof_itf < max_itf) {
        n = read_line(buffer, i, string);
        i += n;
        if (strcmp(string, "[xitf]") == 0) {
            line = 0;
            next_section = 0;
            error = 0;
            while ((!next_section) && (i < file_len)) {
                n = read_line(buffer, i, string);
                if (string[0] == '[')
                    next_section = 1;
                else {
                    i += n;
                    line++;
                    pname = string;
                    pvalue = strchr(string, '=');
                    pvalue[0] = '\0';
                    pvalue++;
                    switch (param_name_start(pname, itf_parameters)) {
                    case ITF_ADDRESS:
                        if (strlen(pvalue) < 16) {
                            strcpy(ext_itf[nof_itf].address, pvalue);
                        } else {
                            printf("Error parsing xitf file. Address to long.\n");
                            error = 1;
                            break;
                        }
                        break;
                    case ITF_ID:
                        sscanf(pvalue, "0x%x", &aux);
                        ext_itf[nof_itf].id = (xitfid_t) aux;
                        ext_itf[nof_itf].info.id = (xitfid_t) aux;
                        break;
                    case ITF_TYPE:
                        if (!strcmp(pvalue,"tcp")) {
                            ext_itf[nof_itf].type=NET_TCP;
                        } else if (!strcmp(pvalue,"udp")) {
                            ext_itf[nof_itf].type=NET_UDP;
                        } else {
                            printf("Error parsing xitf file. Unknown type %s\n",pvalue);
                            error=1;
                            break;
                        }
                        break;
                    case ITF_MODE:
                        if (!strcmp(pvalue, "out"))
                            ext_itf[nof_itf].mode = FLOW_WRITE_ONLY;
                        else if (!strcmp(pvalue, "in"))
                            ext_itf[nof_itf].mode = FLOW_READ_ONLY;
                        else if (!strcmp(pvalue, "inout"))
                            ext_itf[nof_itf].mode = FLOW_READ_WRITE;
                        else if (!strcmp(pvalue, "outin"))
                            ext_itf[nof_itf].mode = FLOW_WRITE_READ;
                        else {
                            printf("Error parsing xitf file. Mode must be in/out.\n");
                            error = 1;
                            break;
                        }
                        ext_itf[nof_itf].info.mode = ext_itf[nof_itf].mode;
                        break;
                    case ITF_PORT:
                        sscanf(pvalue, "%d", &aux);
                        ext_itf[nof_itf].port = aux;
                        break;
                    case ITF_DELAY:
                        ext_itf[nof_itf].info.delay = atof(pvalue);
                        break;
                    case ITF_BW:
                        ext_itf[nof_itf].info.BW = atof(pvalue);
                        break;
                    default:
                        printf("Error parsing xitf file: Unknown parameter\n");
                        error = 1;
                        break;
                    }

                    if (error) {
                        printf("Error parsing xitf file. Itf %d, parameter %d\n", nof_itf + 1, line);
                        return -1;
                    }
                }
            }
            //			ext_itf[nof_itf].type=NET_TCP;
            nof_itf++;
            if (nof_itf == max_itf)
                printf("CAUTION: Maximum interfaces exceed\n");
        }
    }
    if (error) {
        return -1;
    } else
        return nof_itf;
}

/** CPU Configuration Parameters
 */
char *cpu_parameters[NOF_CPU_PARAMS + 1];

int read_daemons(char *pvalue, struct launch_daemons *daemons, int max_d)  {
    char *x,*y;
    int i;

    /** read comma-sepparated values */
    x=pvalue;
    i=0;
    if (x=='\0') {
        return 0;
    }
    do {
        y=x;
        x=strstr(y,",");
        if (x) {
            *x='\0';
            x++;
        }
        strcpy(daemons[i].path,y);
        i++;
    } while(x && i<max_d);

    return i;
}

/** Daemon Configuration Parameters configuration.
 *
 * Configures the parameters accepted in the daemon configuration file
 *
 * @todo Separate CPU parsing from daemon parsing. More CPU parameters?
 * @todo Replace name by daemon code
 *
 */
void init_daemon_param_strings(void)
{
    cpu_parameters[CPU_NAME] = "name";
    cpu_parameters[CPU_MIPS] = "mops";
    cpu_parameters[CPU_TSLOT] = "tslot";
    cpu_parameters[CPU_DAEMON] = "run_as_daemon";
    cpu_parameters[CPU_OUTPUT] = "output_file";
    cpu_parameters[CPU_DEBUG] = "debug_mode";
    cpu_parameters[CPU_RESPATH] = "work_path";
    cpu_parameters[CPU_LOCK] = "lock_file";
    cpu_parameters[CPU_FULLBRIDGE] = "full_bridge";
    cpu_parameters[CPU_ENDIAN] = "endian";
    cpu_parameters[CPU_CORES] = "nof_cores";
    cpu_parameters[CPU_BTATEXIT] = "printbt_atexit";
    cpu_parameters[CPU_BTATTERM] = "printbt_atrtfault";
    cpu_parameters[CPU_DLIST] = "daemons";
    cpu_parameters[CPU_ITFDELAY] = "itfs_delay";
    cpu_parameters[CPU_ARCH] = "arch";
    cpu_parameters[CPU_FLUSH] = "enable_fflush";
    cpu_parameters[CPU_NSAMPLES] = "numsamples";
    cpu_parameters[CPU_ENABLEDAC] = "enable_dac";
    cpu_parameters[CPU_INTBW] = "bw_mbps";
    cpu_parameters[CPU_EXECDIV] = "exec_subslot_div";
    cpu_parameters[NOF_CPU_PARAMS] = "\0";
}

/** Daemon & CPU Configuration Parsing Function.
 *
 * Actually parses the file and fills daemon_db structure with parsed data.
 *
 * @sa init_daemon_param_string
 *
 * @param f FILE pointer
 * @param daemon_db Pointer to first element of the daemon_db database
 * @param cpu_info Pointer to cpu configuration structure
 * @param max_d Size in number of elements of the database
 *
 * @return >=0 Number of daemons parsed
 * @return <0 Error parsing
 */
int parse_cpu_cfg(FILE *f, struct launch_daemons *daemon_db, int max_d, struct hw_api_db *hwapi_db)
{

    int file_len;
    char string[PARSE_LINE_LEN];
    int nof_d;
    int error;
    int next_section, i, n;
    char *buffer, *pname, *pvalue;
    int line;

    init_daemon_param_strings();

    file_len = read_file(f, &buffer, 0);
    if (file_len < 0) {
        printf("ERROR: There were errors reading config file\n");
        return (-1);
    }

    i = 0;
    line = 0;
    error = 0;

    while (i < file_len) {
        n = read_line(buffer, i, string);
        i += n;
        line++;
        pname = string;
        pvalue = strchr(string, '=');
        pvalue[0] = '\0';
        pvalue++;
        switch (param_name_start(pname, cpu_parameters)) {
        case CPU_NAME:
            if (strnlen(pvalue, CPU_NAME_LEN) < CPU_NAME_LEN) {
                strncpy(hwapi_db->cpu_info.name, pvalue, CPU_NAME_LEN);
            } else {
                printf("Error parsing platform file. Processor name too long.\n");
                error = 1;
                break;
            }
            break;
        case CPU_BTATEXIT:
            hwapi_db->printbt_atexit = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_BTATTERM:
            hwapi_db->printbt_atterm = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_ARCH:
            if (!strncmp(pvalue,"x86",3)) {
                hwapi_db->cpu_info.plat_family = PLATFORM_LIN_X86;
            } else if (!strncmp(pvalue,"x86_64",6)) {
                hwapi_db->cpu_info.plat_family = PLATFORM_LIN_X86_64;
            } else if (!strncmp(pvalue,"arm",3)) {
                hwapi_db->cpu_info.plat_family = PLATFORM_LIN_ARM;
            } else if (!strncmp(pvalue,"ppc",3)) {
                hwapi_db->cpu_info.plat_family = PLATFORM_LIN_PPC;
            } else {
                printf("Error parsing platform file. Unknown processor architecture %s\n",pvalue);
                error=1;
                break;
            }
            break;
        case CPU_EXECDIV:
            hwapi_db->exec_slot_div = atoi(pvalue);
            break;
        case CPU_ITFDELAY:
                    hwapi_db->default_delay = atoi(pvalue);
                    break;
        case CPU_INTBW:
            hwapi_db->cpu_info.intBW = atof(pvalue);
            break;
        case CPU_DEBUG:
            hwapi_db->cpu_info.debug_level = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_FLUSH:
            hwapi_db->flushout = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_NSAMPLES:
            hwapi_db->numsamples = atoi(pvalue);            
            break;
        case CPU_ENABLEDAC:
            hwapi_db->enabledac = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_MIPS:
            hwapi_db->cpu_info.C = atof(pvalue);
            break;
        case CPU_TSLOT:
            hwapi_db->cpu_info.tslen_usec = atoi(pvalue);
            break;
        case CPU_DAEMON:
            hwapi_db->run_as_daemon = strncmp(pvalue, "yes", 3) ? 0 : 1;
            break;
        case CPU_OUTPUT:
            strncpy(hwapi_db->output_file, pvalue, DEF_FILE_LEN);
            break;
        case CPU_FULLBRIDGE:
            hwapi_db->net_db.full_bridge = strncmp(pvalue,"yes",3)?0:1;
            break;
        case CPU_ENDIAN:
            hwapi_db->net_db.litendian = strncmp(pvalue,"big",3)?1:0;
            break;
        case CPU_CORES:
            hwapi_db->cpu_info.nof_cores = atoi(pvalue);
            break;
        case CPU_RESPATH:
            strncpy(hwapi_db->res_path, pvalue, DEF_FILE_LEN);
            break;
        case CPU_LOCK:
            strncpy(hwapi_db->lock_file, pvalue, DEF_FILE_LEN);
            break;
        case CPU_DLIST:
            nof_d=read_daemons(pvalue,daemon_db,max_d);
            break;
        default:
            printf("Error parsing daemons file: Unknown parameter %s\n", pname);
            error = 1;
            break;
        }
        if (error) {
            printf("Error Platform's configuration file. Line %d\n", line);
            return -1;
        }
    }
    if (error) {
        return -1;
    } else
        return nof_d;
}
